#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "colas.h"

/**
 * Funcin que crea una cola vaca y reserva memoria para el nodo
 * cabecera
 *
 * Salida:
 *    Estructura Cola con memoria asignada e inicializada
 */
Cola *crearCola()
{
  Cola *C;

  C = (Cola *) malloc(sizeof(Cola));
  if (C == NULL){
    return NULL;
  }

  C->principio = NULL;
  C->final = NULL;

  return C;
}

/**
 * Funcin que libera la memoria que se haba reservado para los datos de la
 * cola
 *
 * Entrada:
 *    Estructura cola a liberar
 */
void borrarCola(Cola *C)
{
  if (C == NULL)
    printf("Error: primero hay que crear la cola\n");
  else {
    while(!esVaciaCola(*C)) {
      desencolar(C);
    }
    free(C);
  }
}

/**
 * Comprueba si la cola C esta vaca 
 *
 * Entrada:
 *    C: Cola que se quiere comprobar
 *
 * Salida:
 *    Devuelve 1 si lacola esta vaca y 0 si tiene al menos un elemento
 */
int esVaciaCola(Cola C)
{
  return(C.principio == NULL);
}

/**
 * Aade un elemento a la cola
 *
 * Entrada:
 *    X: Elemento a aadir
 *    C: Puntero a la cola
 *
 * Salida:
 *   Devuelve 1 si no ha podido aadir el elemento y 0 en caso contrario 
 */
int encolar(int X, Cola *C)
{
  PNodoC NodoTemp;

  NodoTemp = (PNodoC) malloc(sizeof(NodoC));
  if (NodoTemp == NULL){
    printf("Error de asignacin: No hay espacio");
    return 1;
  }

  NodoTemp->dato = X;
  NodoTemp->siguiente = NULL;

  if (esVaciaCola(*C))
    C->principio = NodoTemp;
  else
    C->final->siguiente = NodoTemp;

  C->final = NodoTemp;

  return 0;
}

/**
 * Elimina el elemento que lleva ms esperando de la cola
 *
 * Entrada:
 *   C: Cola de la que se quiere eliminar
 *
 * Salida:
 *    
 */
void desencolar(Cola *C)
{
  PNodoC primero;

  if (esVaciaCola(*C))
    printf("Cola Vaca\n");
  else {
    primero = C->principio;
    C->principio = C->principio->siguiente;
    if (C->principio == NULL)
      C->final = NULL;
    free(primero);
  }
}

/**
 * Te devuelve el valor del elemento primero de la cola
 *
 * Entrada:
 *    C: Cola de la que se quiere obtener el primero
 *
 * Salida:
 *    Devuelve el valor encontrado
 */
int primero(Cola C)
{
  assert(!esVaciaCola(C));

  return C.principio->dato;
}

/**
 * Funcin para imprimir una cola
 *
 * Entrada:
 *    C: cola a imprimir
 */
void imprimirCola(Cola *C)
{
  PNodoC N;
  N = C->principio;
  while (N != NULL) {
    printf("%d ",N->dato);
    N = N -> siguiente;
  }
  printf("\n");
}

