/**
 * Fichero : colas.h      Fecha: 01/02/2019
 * Autores:
 * 
 *  TAD para crear colas de enteros
 *                                            
 */

#ifndef __COLAS_H_
#define __COLAS_H_

/* Estructura del TAD cola */
typedef struct NodoC *PNodoC;

typedef struct NodoC {
	int dato;
	PNodoC siguiente;
} NodoC;

typedef struct Cola {
	PNodoC principio;
	PNodoC final;
} Cola;


/**
 * Función que crea una cola vacía y reserva memoria para el nodo
 * cabecera
 *
 * Salida:
 *    Estructura Cola con memoria asignada e inicializada
 */
Cola *crearCola();

/**
 * Función que libera la memoria que se había reservado para los datos de la
 * cola
 *
 * Entrada:
 *    Estructura cola a liberar
 */
void borrarCola(Cola *C);

/**
 * Comprueba si la cola C esta vacía 
 *
 * Entrada:
 *    C: Cola que se quiere comprobar
 *
 * Salida:
 *    Devuelve 1 si lacola esta vacía y 0 si tiene al menos un elemento
 */
int esVaciaCola(Cola C);

/**
 * Añade un elemento a la cola
 *
 * Entrada:
 *    X: Elemento a añadir
 *    C: Puntero a la cola
 *
 * Salida:
 *   Devuelve 1 si no ha podido añadir el elemento y 0 en caso contrario 
 */
int encolar(int X, Cola *C);

/**
 * Elimina el elemento que lleva más esperando de la cola
 *
 * Entrada:
 *   C: Cola de la que se quiere eliminar
 *
 * Salida:
 *    
 */
void desencolar(Cola *C);

/**
 * Te devuelve el valor del elemento primero de la cola
 *
 * Entrada:
 *    C: Cola de la que se quiere obtener el primero
 *
 * Salida:
 *    Devuelve el valor encontrado
 */
int primero(Cola C);


/**
 * Función para imprimir una cola
 *
 * Entrada:
 *    C: cola a imprimir
 */
void imprimirCola(Cola *C);

#endif

