#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "colasP.h"

/**
 * Funcin que crea una cola vaca y reserva memoria para el nodo
 * cabecera
 *
 * Salida:
 *    Estructura ColaP con memoria asignada e inicializada
 */
ColaP *crearColaP()
{
  ColaP *C;

  C = (ColaP *) malloc(sizeof(ColaP));
  if (C == NULL){
    return NULL;
  }

  C->principio = NULL;
  C->final = NULL;

  return C;
}

/**
 * Funcin que libera la memoria que se haba reservado para los datos de la
 * cola
 *
 * Entrada:
 *    Estructura cola a liberar
 */
void borrarColaP(ColaP *C)
{
  if (C == NULL)
    printf("Error: primero hay que crear la cola\n");
  else {
    while(!esVaciaColaP(*C)) {
      desencolarP(C);
    }
    free(C);
  }
}

/**
 * Comprueba si la cola C esta vaca 
 *
 * Entrada:
 *    C: ColaP que se quiere comprobar
 *
 * Salida:
 *    Devuelve 1 si lacola esta vaca y 0 si tiene al menos un elemento
 */
int esVaciaColaP(ColaP C)
{
  return(C.principio == NULL);
}

/**
 * Aade un elemento a la cola
 *
 * Entrada:
 *    X: Elemento a aadir
 *    C: Puntero a la cola
 *
 * Salida:
 *   Devuelve 1 si no ha podido aadir el elemento y 0 en caso contrario 
 */
int encolarP(void *dato, ColaP *C)
{
  PNodoCP NodoTemp;

  NodoTemp = (PNodoCP) malloc(sizeof(NodoCP));
  if (NodoTemp == NULL){
    printf("Error de asignacin: No hay espacio");
    return 1;
  }

  NodoTemp->dato = dato;
  NodoTemp->siguiente = NULL;
  if (esVaciaColaP(*C))
    C->principio = NodoTemp;
  else
    C->final->siguiente = NodoTemp;
  C->final = NodoTemp;

  return 0;
}

/**
 * Elimina el elemento que lleva ms esperando en la cola
 *
 * Entrada:
 *   C: ColaP de la que se quiere eliminar
 *
 * Salida:
 *    
 */
void desencolarP(ColaP *C)
{
  PNodoCP Primero;

  if (esVaciaColaP(*C))
    printf("ColaP Vaca\n");
  else {
    Primero = C->principio;
    C->principio = C->principio->siguiente;
    if (C->principio == NULL)
      C->final = NULL;
    free(Primero);
  }

}

/**
 * Te devuelve el valor del elemento primero de la cola
 *
 * Entrada:
 *    C: ColaP de la que se quiere obtener el primero
 *
 * Salida:
 *    Devuelve el valor encontrado
 */
void* primeroP(ColaP C)
{
  assert(!esVaciaColaP(C));

  return C.principio->dato;
}

