/**
 * Fichero : colasP.h      Fecha: 01/02/2019
 * Autores:
 * 
 *  TAD para crear colas de punteros
 *                                            
 */

#ifndef __COLASP_H_
#define __COLASP_H_

/* Estructura del TAD cola */
typedef struct NodoCP *PNodoCP;

typedef struct NodoCP {
	void* dato;
	PNodoCP siguiente;
} NodoCP;

typedef struct ColaP {
	PNodoCP principio;
	PNodoCP final;
} ColaP;


/**
 * Función que crea una cola vacía y reserva memoria para el nodo
 * cabecera
 *
 * Salida:
 *    Estructura ColaP con memoria asignada e inicializada
 */
ColaP *crearColaP();

/**
 * Función que libera la memoria que se había reservado para los datos de la
 * cola
 *
 * Entrada:
 *    Estructura cola a liberar
 */
void borrarColaP(ColaP *C);

/**
 * Comprueba si la cola C esta vacía 
 *
 * Entrada:
 *    C: ColaP que se quiere comprobar
 *
 * Salida:
 *    Devuelve 1 si lacola esta vacía y 0 si tiene al menos un elemento
 */
int esVaciaColaP(ColaP C);

/**
 * Añade un elemento a la cola
 *
 * Entrada:
 *    X: Elemento a añadir
 *    C: Puntero a la cola
 *
 * Salida:
 *   Devuelve 1 si no ha podido añadir el elemento y 0 en caso contrario 
 */
int encolarP(void* X, ColaP *C);

/**
 * Elimina el elemento que lleva más esperando en la cola
 *
 * Entrada:
 *   C: ColaP de la que se quiere eliminar
 *
 * Salida:
 *    
 */
void desencolarP(ColaP *C);

/**
 * Te devuelve el valor del elemento primero de la cola
 *
 * Entrada:
 *    C: ColaP de la que se quiere obtener el primero
 *
 * Salida:
 *    Devuelve el valor encontrado
 */
void* primeroP(ColaP C);


#endif

