/**
 * Fichero : listas.c      Fecha: 01/02/2019
 * 
 *  TAD para crear listas enlazadas de enteros
 *                                            
 */

#include <stdlib.h>
#include <stdio.h>
#include "listas.h"

/**
 * Función que crea una lista enlazada vacía y reserva memoria para el nodo
 * cabecera
 *
 * Salida:
 *    Estructura Lista con memoria asignada e inicializada
 */
Lista crearLista()
{
  Lista L;

  L = (PNodo) malloc(sizeof(Nodo));
  if (L == NULL){
    return NULL;
  }
  
  L->siguiente = NULL;

  return L;
}

/**
 * Función que libera la memoria que se había reservado para los datos de la
 * lista
 *
 * Entrada:
 *    Estructura lista a liberar
 */
void borrarLista(Lista L)
{
  PNodo P, Temp;

  P = L->siguiente; /* Supone cabecera */
  L->siguiente = NULL;
  while(P != NULL){
    Temp = P->siguiente;
    free(P);
    P = Temp;
  }

  free(L);
}

/**
 * Inserta después del nodo apuntado por P. Supone una implementación con
 * cabecera. 
 *
 * Entrada:
 *    X: Elemento a añadir
 *    P: Nodo tras el que se inserta X
 */
int insertar(int X, PNodo P)
{
  PNodo NodoTemp;

  NodoTemp = malloc(sizeof(Nodo));
  if (NodoTemp == NULL){
    printf("Error de asignación: No hay espacio");
    return 1;
  }

  NodoTemp->dato = X;
  NodoTemp->siguiente = P->siguiente;
  P->siguiente = NodoTemp;

  return 0;
}

/**
 * Función para imprimir la lista
 *
 * Entrada:
 *    L: Lista a imprimir
 */
void imprimirLista(Lista L)
{
  PNodo P;
  
  P = L->siguiente;
  while (P != NULL) {
    printf("%d ",P->dato);
    P = P -> siguiente;
  }
  printf("\n");
}

/**
 * Devuelve un puntero al nodo que contiene X como dato; NULL si no lo encuentra 
 *
 * Entrada:
 *    X: Elemento a buscar
 *    L: Puntero a la lista
 */
PNodo buscarElemento(int X, Lista L)
{
  PNodo P;

  P = L->siguiente;
  while (P != NULL && P->dato != X){
    P = P->siguiente;
  }

  return P;
}

/**
 * Comprueba si la lista L esta vacía 
 *
 * Entrada:
 *    L: Lista que se quiere comprobar
 *
 * Salida:
 *    Devuelve 1 si la lista esta vacía y 0 si tiene al menos un elemento
 */
int esVacia(Lista L)
{
  return(L->siguiente == NULL);
}

/**
 * Comprueba si el nodo P es el último  
 *
 * Entrada:
 *    P: Nodo sobre el que se quiere comprobar si es el último
 *
 * Salida:
 *    Devuelve 1 si el nodo es el último y 0 si no
 */
int esUltimo(PNodo P)
{
  return P->siguiente == NULL;
}

/**
 *  Devuelve un puntero al nodo anterior al que contiene X como dato
 * 
 * Entrada:
 *    X: Dato del que se quiere obtener el nodo anterior
 *    L: Lista de trabajo
 *
 * Salida:
 *    Devuelve el nodo encontrado. Si no se encuentra X devuelve el último nodo.
 *    Si la lista esta vacía, devuelve un puntero al nodo cabecera
 */
PNodo buscarAnterior(int X, Lista L)
{
  PNodo P;
  P = L;

  while (P->siguiente != NULL 
    && P->siguiente->dato != X){
    P = P->siguiente;
  }

  return P;
}

/**
 * Borra el nodo de la lista L que contiene el dato X y no hace nada si X no
 * está en la lista enlazada
 *
 * Entrada:
 *    X: Dato del que se quiere borrar el nodo
 *    L: Lista de trabajo
 */
void borrar(int X, Lista L)
{
  PNodo P, NodoTemp;

  P = buscarAnterior(X,L); /*Devuelve último si X no está en L */
  if (!esUltimo(P)){   
    NodoTemp = P->siguiente;
    P->siguiente = NodoTemp->siguiente;
    free(NodoTemp); 
  }
}

