/**
 * Fichero : listas.h      Fecha: 01/02/2019
 * Autores:
 * 
 *  TAD para crear listas enlazadas con enteros
 *                                            
 */

#ifndef __LISTAS_H_
#define __LISTAS_H_

/* Estructura del TAD lista enlazada y nodo de la lista*/
typedef struct Nodo *PNodo;
typedef struct Nodo {
	int dato;
	PNodo siguiente;
} Nodo;

typedef struct Nodo *Lista;

/**
 * Función que crea una lista enlazada vacía y reserva memoria para el nodo
 * cabecera
 *
 * Salida:
 *    Estructura Lista con memoria asignada e inicializada
 */
Lista crearLista();

/**
 * Función que libera la memoria que se había reservado para los datos de la
 * lista
 *
 * Entrada:
 *    Estructura lista a liberar
 */
void borrarLista(Lista L);

/**
 * Inserta después del nodo apuntado por P. Supone una implementación con
 * cabecera. 
 *
 * Entrada:
 *    X: Elemento a añadir
 *    P: Nodo tras el que se inserta X
 */
int insertar(int X, PNodo P);

/**
 * Función para imprimir la lista
 *
 * Entrada:
 *    L: Lista a imprimir
 */
void imprimirLista(Lista L);

/**
 * Devuelve un puntero al nodo que contiene X como dato; NULL si no lo encuentra 
 *
 * Entrada:
 *    X: Elemento a buscar
 *    L: Puntero a la lista
 */
PNodo buscarElemento(int X, Lista L);

/**
 * Comprueba si la lista L esta vacía 
 *
 * Entrada:
 *    L: Lista que se quiere comprobar
 *
 * Salida:
 *    Devuelve 1 si la lista esta vacía y 0 si tiene al menos un elemento
 */
int esVacia(Lista L);

/**
 * Comprueba si el nodo P es el último  
 *
 * Entrada:
 *    P: Nodo sobre el que se quiere comprobar si es el último
 *
 * Salida:
 *    Devuelve 1 si el nodo es el último y 0 si no
 */
int esUltimo(PNodo P);

/**
 *  Devuelve un puntero al nodo anterior al que contiene X como dato
 * 
 * Entrada:
 *    X: Dato del que se quiere obtener el nodo anterior
 *    L: Lista de trabajo
 *
 * Salida:
 *    Devuelve el nodo encontrado. Si no se encuentra X devuelve el último nodo.
 *    Si la lista esta vacía, devuelve un puntero al nodo cabecera
 */
PNodo buscarAnterior(int X, Lista L);

/**
 * Borra el nodo de la lista L que contiene el dato X y no hace nada si X no
 * está en la lista enlazada
 *
 * Entrada:
 *    X: Dato del que se quiere borrar el nodo
 *    L: Lista de trabajo
 */
void borrar(int X, Lista L);

#endif

