/**
 * Fichero : pilas.c      Fecha: 01/02/2019
 * Autores:
 * 
 *  TAD para crear pilas de enteros
 *                                            
 */

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "pilas.h"

/**
 * Funcin que crea una pila vaca y reserva memoria para el nodo
 * cabecera
 *
 * Salida:
 *    Estructura Pila con memoria asignada e inicializada
 */
Pila crearPila()
{
  Pila P;

  P = (PNodoP) malloc(sizeof(NodoP));
  if (P == NULL){
    return NULL;
  }
  
  P->siguiente = NULL;

  return P;
}

/**
 * Funcin que libera la memoria que se haba reservado para los datos de la
 * pila
 *
 * Entrada:
 *    Estructura pila a liberar
 */
void borrarPila(Pila P)
{
  if (P == NULL) {
    printf("Error: primero hay que crear la pila\n");
  }
  else {
    while(!esVaciaPila(P)) {
      desapilar(P);
    }
    free(P);
  }
}

/**
 * Aade un elemento a la pila
 *
 * Entrada:
 *    X: Elemento a aadir
 *    P: Puntero a la pila
 *
 * Salida:
 *   Devuelve 1 si no ha podido aadir el elemento y 0 en caso contrario 
 */
int apilar(int X, Pila P)
{
  PNodoP NodoTemp;

  NodoTemp = (PNodoP) malloc(sizeof(NodoP));
  if (NodoTemp == NULL){
    printf("Error de asignacin: No hay espacio");
    return 1;
  }

  NodoTemp->dato = X;
  NodoTemp->siguiente = P->siguiente;
  P->siguiente = NodoTemp;

  return 0;
}

/**
 * Funcin para imprimir una pila
 *
 * Entrada:
 *    P: Pila a imprimir
 */
void imprimirPila(Pila P)
{
  PNodoP N;
  
  N = P->siguiente;
  while (N != NULL) {
    printf("%d ",N->dato);
    N = N -> siguiente;
  }
  printf("\n");
}

/**
 * Comprueba si la pila P esta vaca 
 *
 * Entrada:
 *    P: Pila que se quiere comprobar
 *
 * Salida:
 *    Devuelve 1 si la pila esta vaca y 0 si tiene al menos un elemento
 */
int esVaciaPila(Pila P)
{
  return(P->siguiente == NULL);
}


/**
 * Elimina el elemento en la cima de la pila
 *
 * Entrada:
 *    P: Pila de la que se quiere eliminar la cima
 *
 * Salida:
 *    
 */
void desapilar(Pila P)
{
  PNodoP Primero;

  if (esVaciaPila(P))
    printf("Pila Vaca\n");
  else {
    Primero = P->siguiente;
    P->siguiente = P->siguiente->siguiente;
    free(Primero); 
  }
}

/**
 * Te devuelve el valor del elemento en la cima de la pila
 *
 * Entrada:
 *    P: Pila de la que se quiere obtener la cima
 *
 * Salida:
 *    Devuelve el valor encontrado
 */
int cima(Pila P)
{
  assert(!esVaciaPila(P));

  return P->siguiente->dato;
}

/**
 * Elimina el elemento n-esimo de la pila
 *
 * Entrada:
 *    P: Pila de la que se quiere eliminar la cima
 *
 * Salida:
 *    
 */
void borrar_enesimo(Pila P, int n)
{
  Pila Aux;
  int dato;
  int i;

  i = 1;

  Aux = crearPila();
  while ((!esVaciaPila(P)) && (i<n)) {
    dato = cima(P);
    printf("dato %d\n",dato);
    apilar(dato,Aux);
    desapilar(P);
    i = i+1;
  }
  desapilar(P);
  while (!esVaciaPila(Aux)) {
    dato = cima(Aux);
    apilar(dato,P);
    desapilar(Aux);
  }
  borrarPila(Aux);
}


