/**
 * Fichero : pilas.h      Fecha: 01/02/2019
 * Autores:
 * 
 *  TAD para crear pilas de enteros
 *                                            
 */

#ifndef __PILAS_H_
#define __PILAS_H_

/* Estructura del TAD pila */
typedef struct NodoP *PNodoP;

typedef struct NodoP {
	int dato;
	PNodoP siguiente;
} NodoP;

typedef struct NodoP *Pila;

/**
 * Función que crea una pila vacía y reserva memoria para el nodo
 * cabecera
 *
 * Salida:
 *    Estructura Pila con memoria asignada e inicializada
 */
Pila crearPila();

/**
 * Función que libera la memoria que se había reservado para los datos de la
 * pila
 *
 * Entrada:
 *    Estructura pila a liberar
 */
void borrarPila(Pila P);

/**
 * Añade un elemento a la pila
 *
 * Entrada:
 *    X: Elemento a añadir
 *    P: Puntero a la pila
 *
 * Salida:
 *   Devuelve 1 si no ha podido añadir el elemento y 0 en caso contrario 
 */
int apilar(int X, Pila P);

/**
 * Función para imprimir una pila
 *
 * Entrada:
 *    P: Pila a imprimir
 */
void imprimirPila(Pila P);

/**
 * Comprueba si la pila P esta vacía 
 *
 * Entrada:
 *    P: Pila que se quiere comprobar
 *
 * Salida:
 *    Devuelve 1 si la pila esta vacía y 0 si tiene al menos un elemento
 */
int esVaciaPila(Pila P);

/**
 * Elimina el elemento en la cima de la pila
 *
 * Entrada:
 *    P: Pila de la que se quiere eliminar la cima
 *
 * Salida:
 *    
 */
void desapilar(Pila P);

/**
 * Te devuelve el valor del elemento en la cima de la pila
 *
 * Entrada:
 *    P: Pila de la que se quiere obtener la cima
 *
 * Salida:
 *    Devuelve el valor encontrado
 */
int cima(Pila P);

/**
 * Elimina el elemento n-esimo de la pila
 *
 * Entrada:
 *    P: Pila de la que se quiere eliminar la cima
 *
 * Salida:
 *    
 */
void borrar_enesimo(Pila P, int n);

#endif






